; Base code for CGB
; (c) 1999 ACZ

;rst equates
_ldhlind			=		$00			;ld hl,(hl)
_addhla				=		$08			;add hl,a
_cphlde				=		$10			;cp hl,de
_subhlde			=		$18			;sub hl,de (not sbc hl,de)
_exdehl				=		$20			;ex de,hl
_cpagejp			= 		$28			;cross-page jp
_cpagecall			=		$30			;cross-page call
_cpageldhl			= 		$38			;cross-page ld hl,*

_main				=		$150		;program start
_vblank				=		$ff83		;vblank interrupt handler

Base:

.pad _ldhlind			;ld hl,(hl)
 push af
 ld a,(hl)
 inc hl
 ld l,(hl)
 ld h,a
 pop af
 ret

.pad _addhla			;add hl,a
 add a,l
 ld l,a
 ret nc
 inc h
 ret

.pad _cphlde			;cp hl,de
 ld a,h
 xor d
 ret nz
 ld a,l
 xor e
 ret

.pad _subhlde			;sub hl,de
 ld a,l
 sub e
 ld a,h
 sbc a,d
 ret

.pad _exdehl			;ex de,hl
 ld a,l
 sub e
 ld a,h
 sbc a,d
 ret

.pad _cpagejp			;cross-page jp
 call @cpagerst
 jr @cpagejp

.pad _cpagecall			;cross-page call
 call @cpagerst
 jr @cpagecall

.pad _cpageldhl			;cross-page ld hl,*
 call @cpagerst
 jr @cpageldhl

.pad $40				;VBL
 jp _vblank

.pad $48				;disable LCDC status int
 reti

.pad $50				;disable timer overflow
 reti

.pad $58				;disable serial transfer int
 reti

.pad $60				;disable keypad oscillation int
 reti

@cpagerst:
 push af
 push hl
 ldhl sp,6				; point to address on stack
 ldi a,(hl)				; load lsb of address
 ld h,(hl)				; load msb of address
 ld l,a					; complete pointer  
 inc hl					; past RST instr
 ldi a,(hl)				; get lsb of address
 ldh ($81),a			; load into high ram
 ldi a,(hl)				; get msb of address
 ldh ($82),a			; load into high ram
 ld a,(hl)				; get page to switch to
 ld ($2000),a			; switch rom pages
 pop hl
 pop af
 ret

@cpagejp:
 add sp,2				; clear return address
 push af
 ld a,$c3				; opcode for JP
 ldh ($80),a			; load into high ram
 pop af
 jp $ff80				; jump to instruction in high ram

@cpagecall:
 push af
 ld a,$cd				; opcode for CALL
 ldh ($80),a			; load into high ram
 pop af
 jp $ff80				; jump to instruction in high ram

@cpageldhl:
 push af
 ldh a,($81)			; load from high ram
 ld l,a					; into lsb
 ldh a,($82)			; load from high ram
 ld h,a					; into msb
 pop af
 ret					; back to caller

.pad $100
 nop
 jp _main

;scrolling nintendo graphic
.db $CE,$ED,$66,$66,$CC,$0D,$00,$0B,$03,$73,$00,$83,$00,$0C,$00,$0D
.db $00,$08,$11,$1F,$88,$89,$00,$0E,$DC,$CC,$6E,$E6,$DD,$DD,$D9,$99
.db $BB,$BB,$67,$63,$6E,$0E,$EC,$CC,$DD,$DC,$99,$9F,$BB,$B9,$33,$3E

.db "FALL DOWN"		;15 bytes
.db 0,0,0,0,0,0
.db $80				;gameboy color
.dw 0				;licensee code
.db 0				;gameboy
.db $1b				;ROM+MBC5+RAM+BATT
.db $00				;ROM Size (set by linker)
.db $02				;RAM Size (8kb)
.db 1				;non-japanese
.db $33				;check $144/$145 for license code
.db $00				;mask rom version #
.db $ff				;complement check
.db $ff,$ff       	;checksum

.end
